import { noop, safe_not_equal } from "svelte/internal";
export const Refuse = Symbol();
export function objectAccessor(key) {
    return {
        readChild: (parent) => parent[key],
        writeChild: (parent, newChild) => (parent[key] = newChild),
    };
}
export function mapAccessor(key) {
    return {
        readChild: (parent) => 
        /* eslint-disable-next-line @typescript-eslint/no-non-null-assertion */
        parent.has(key) ? parent.get(key) : Refuse,
        writeChild: (parent, newChild) => {
            parent.set(key, newChild);
        },
    };
}
const subscriber_queue = [];
export function readableTree(value, start = noop) {
    const { subscribe, zoom, zoomWritable, zoomIn, zoomInWritable, choose, chooseWritable, } = writableTree(value, start);
    return {
        subscribe,
        zoom,
        zoomWritable,
        zoomIn,
        zoomInWritable,
        choose,
        chooseWritable,
    };
}
export function writableTree(value, start = noop) {
    const tree = {
        children: new Set(),
        thisSubscribers: new Set(),
    };
    let countAllSubscribers = 0;
    let stop;
    return writableTreeCore(() => value, (newValue) => (value = newValue), tree, function incrementThenStartIfNecessary(set) {
        ++countAllSubscribers;
        if (countAllSubscribers === 1) {
            stop = start(set) || noop;
        }
    }, function decrementThenStopIfNecessary() {
        --countAllSubscribers;
        if (countAllSubscribers === 0) {
            stop();
            stop = undefined;
        }
    }, function isReady() {
        return !!stop;
    }, []);
}
function writableTreeCore(getValue, writeValue, tree, incrementThenStartIfNecessary, decrementThenStopIfNecessary, isReady, parentSubscribers) {
    function set(new_value) {
        if (safe_not_equal(getValue(), new_value)) {
            writeValue(new_value);
            if (isReady()) {
                // store is ready
                const run_queue = !subscriber_queue.length;
                for (const parentSubscriber of parentSubscribers) {
                    parentSubscriber(subscriber_queue);
                }
                for (const subscriber of tree.thisSubscribers) {
                    subscriber.i();
                    const chosenValue = subscriber.g();
                    if (chosenValue !== Refuse) {
                        subscriber_queue.push(subscriber, chosenValue);
                    }
                }
                const tmpQueue = [];
                for (const getterAndChild of tree.children.values()) {
                    tmpQueue.push(getterAndChild);
                }
                let getterAndChild;
                while ((getterAndChild = tmpQueue.shift())) {
                    for (const subscribersNode of getterAndChild.c.thisSubscribers) {
                        subscribersNode.i();
                        const chosenChild = subscribersNode.g();
                        if (chosenChild !== Refuse) {
                            subscriber_queue.push(subscribersNode, chosenChild);
                        }
                    }
                    for (const childNode of getterAndChild.c.children.values()) {
                        tmpQueue.push(childNode);
                    }
                }
                if (run_queue) {
                    for (let i = 0; i < subscriber_queue.length; i += 2) {
                        subscriber_queue[i].r(subscriber_queue[i + 1]);
                    }
                    subscriber_queue.length = 0;
                }
            }
        }
    }
    function update(fn) {
        const v = getValue();
        if (v !== Refuse) {
            set(fn(v));
        }
    }
    function subscribe(run, invalidate = noop) {
        const subscriber = {
            r: run,
            i: invalidate,
            g: getValue,
        };
        tree.thisSubscribers.add(subscriber);
        incrementThenStartIfNecessary(set);
        const v = getValue();
        if (v !== Refuse) {
            run(v);
        }
        return () => {
            tree.thisSubscribers.delete(subscriber);
            decrementThenStopIfNecessary();
        };
    }
    function zoom(accessor) {
        const { subscribe, zoom, zoomWritable: zw, zoomIn, zoomInWritable, choose, chooseWritable, } = zoomWritable(accessor);
        return {
            subscribe,
            zoom,
            zoomWritable: zw,
            zoomIn,
            zoomInWritable,
            choose,
            chooseWritable,
        };
    }
    function zoomWritable(accessor) {
        const getter = () => {
            const v = getValue();
            if (v === Refuse) {
                return Refuse;
            }
            return accessor.readChild(v);
        };
        const child = {
            g: getter,
            c: {
                children: new Set(),
                thisSubscribers: new Set(),
            },
        };
        tree.children.add(child);
        return writableTreeCore(getter, (newChild) => {
            const v = getValue();
            if (v === Refuse) {
                return;
            }
            accessor.writeChild(v, newChild);
        }, child.c, function incrementThenStartIfNecessaryForChild(set) {
            incrementThenStartIfNecessary((parent) => {
                const v = accessor.readChild(parent);
                if (v !== Refuse) {
                    set(v);
                }
            });
        }, decrementThenStopIfNecessary, isReady, [
            ...parentSubscribers,
            (subscriber_queue) => {
                for (const subscriber of tree.thisSubscribers) {
                    const chosenValue = subscriber.g();
                    if (chosenValue !== Refuse) {
                        subscriber.i();
                        subscriber_queue.push(subscriber, chosenValue);
                    }
                }
            },
        ]);
    }
    function zoomIn(k) {
        return zoom(objectAccessor(k));
    }
    function zoomInWritable(k) {
        return zoomWritable(objectAccessor(k));
    }
    function choose(chooseParent) {
        const { subscribe, zoom, zoomWritable, zoomIn, zoomInWritable, choose, chooseWritable: cw, } = chooseWritable(chooseParent);
        return {
            subscribe,
            zoom,
            zoomWritable,
            zoomIn,
            zoomInWritable,
            choose,
            chooseWritable: cw,
        };
    }
    function chooseWritable(chooseParent) {
        return writableTreeCore(function newGetValue() {
            const v = getValue();
            if (v === Refuse) {
                return Refuse;
            }
            return chooseParent(v);
        }, writeValue, tree, incrementThenStartIfNecessary, decrementThenStopIfNecessary, isReady, parentSubscribers);
    }
    return {
        set,
        update,
        subscribe,
        zoom,
        zoomWritable,
        zoomIn,
        zoomInWritable,
        choose,
        chooseWritable,
    };
}
//# sourceMappingURL=index.js.map